"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.astToUnistyles = astToUnistyles;
var _functional = require("./functional.js");
const cssPropertiesToRnProps = {
  'background-color': 'backgroundColor',
  'border-width': 'borderWidth',
  'border-style': 'borderStyle',
  'border-color': 'borderColor',
  'border-radius': 'borderRadius',
  'border-top-left-radius': 'borderTopLeftRadius',
  'border-top-right-radius': 'borderTopRightRadius',
  'border-bottom-left-radius': 'borderBottomLeftRadius',
  'border-bottom-right-radius': 'borderBottomRightRadius',
  'border-top-width': 'borderTopWidth',
  'border-bottom-width': 'borderBottomWidth',
  'border-left-width': 'borderLeftWidth',
  'border-right-width': 'borderRightWidth',
  'border-top-style': 'borderTopStyle',
  'border-bottom-style': 'borderBottomStyle',
  'border-left-style': 'borderLeftStyle',
  'border-right-style': 'borderRightStyle',
  'border-top-color': 'borderTopColor',
  'border-bottom-color': 'borderBottomColor',
  'border-left-color': 'borderLeftColor',
  'border-right-color': 'borderRightColor',
  'border-top-left-color': 'borderTopLeftColor',
  'border-top-right-color': 'borderTopRightColor',
  'border-bottom-left-color': 'borderBottomLeftColor',
  'border-bottom-right-color': 'borderBottomRightColor',
  'padding-top': 'paddingTop',
  'padding-bottom': 'paddingBottom',
  'padding-left': 'paddingLeft',
  'padding-right': 'paddingRight',
  'margin-top': 'marginTop',
  'margin-bottom': 'marginBottom',
  'margin-left': 'marginLeft',
  'margin-right': 'marginRight',
  'font-size': 'fontSize',
  'font-weight': 'fontWeight',
  'font-family': 'fontFamily',
  'font-style': 'fontStyle',
  'font-variant': 'fontVariant',
  'font-variant-ligatures': 'fontVariantLigatures',
  'font-variant-caps': 'fontVariantCaps',
  'text-align': 'textAlign',
  'text-transform': 'textTransform',
  'text-decoration': 'textDecoration',
  'text-decoration-color': 'textDecorationColor',
  'text-decoration-style': 'textDecorationStyle',
  'text-decoration-line': 'textDecorationLine',
  'padding': 'padding',
  'color': 'color',
  'flex': 'flex',
  'flex-direction': 'flexDirection',
  'flex-wrap': 'flexWrap',
  'flex-grow': 'flexGrow',
  'flex-shrink': 'flexShrink',
  'flex-basis': 'flexBasis',
  'align-items': 'alignItems',
  'justify-content': 'justifyContent',
  'height': 'height',
  'width': 'width',
  'fill': 'fill',
  'stroke': 'stroke',
  'stroke-width': 'strokeWidth',
  'stroke-linecap': 'strokeLinecap',
  'stroke-linejoin': 'strokeLinejoin',
  'stroke-dasharray': 'strokeDasharray',
  'stroke-dashoffset': 'strokeDashoffset',
  'position': 'position',
  'top': 'top',
  'left': 'left',
  'right': 'right',
  'bottom': 'bottom',
  'z-index': 'zIndex',
  'gap': 'gap',
  'row-gap': 'rowGap',
  'column-gap': 'columnGap',
  'overflow': 'overflow',
  'overflow-x': 'overflowX',
  'overflow-y': 'overflowY',
  'display': 'display',
  'align-self': 'alignSelf',
  'margin-inline': 'marginInline',
  'max-width': 'maxWidth',
  'max-height': 'maxHeight'
};
function parserBorder(border) {
  const [width, style, color] = border.split(' ');
  return {
    borderWidth: parseValue(width),
    borderStyle: style,
    borderColor: color
  };
}
function parsePadding(input) {
  // divide por qualquer espaço / quebra de linha e remove vazios
  const parts = input.trim().split(/\s+/);

  // garante 1‑4 valores; se vier >4, consideramos só os 4 primeiros
  const [a, b = a, c = a, d = b] = parts.slice(0, 4);

  // helper parseValue() já cuida de "px", "%", number etc.
  return {
    paddingTop: parseValue(a),
    paddingRight: parseValue(b),
    paddingBottom: parseValue(c),
    paddingLeft: parseValue(d)
  };
}
function parseValue(value) {
  if (value == null) return undefined;
  const v = value.trim();

  // px → número
  if (/^-?\d+(\.\d+)?px$/.test(v)) {
    return parseFloat(v);
  }

  // percentual → string
  if (/^-?\d+(\.\d+)?%$/.test(v)) {
    return v;
  }

  // número puro → número
  if (/^-?\d+(\.\d+)?$/.test(v)) {
    return parseFloat(v);
  }

  // fallback (cores, palavras‑chave, unidades não suportadas etc)
  return v;
}
function astToUnistyles(ast) {
  let stylesheet = {};
  for (const node of ast.body) {
    if (node.type === 'Declaration') {
      (0, _functional.Functional)(node).property('property', {
        is: 'border'
      }, () => {
        const border = parserBorder(node.value);
        stylesheet = {
          ...stylesheet,
          ...border
        };
      }).property('property', {
        is: 'padding'
      }, () => {
        const padding = parsePadding(node.value);
        stylesheet = {
          ...stylesheet,
          ...padding
        };
      }).continue(() => {
        const property = cssPropertiesToRnProps[node.property];
        const style = {
          [property]: parseValue(node.value)
        };
        stylesheet = {
          ...stylesheet,
          ...style
        };
      });
    }
  }
  return stylesheet;
}
//# sourceMappingURL=ast-to-unistyles.js.map