"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseTokensToAST = parseTokensToAST;
/*************************************************
 * 1) Definições de tipos para TOKENS
 ************************************************/

/*************************************************
 * 2) Definições de tipos para a AST
 ************************************************/

/*************************************************
 * 3) Parser principal
 ************************************************/

function parseTokensToAST(tokens) {
  const ast = {
    type: 'StyleSheet',
    body: []
  };
  let current = 0;
  const peek = () => tokens[current];
  const consume = () => tokens[current++];
  const isAtEnd = () => current >= tokens.length;
  while (!isAtEnd()) {
    const node = parseNode();
    if (node) ast.body.push(node);
  }
  return ast;
  function parseNode() {
    skipPunctuation(';');
    if (isAtEnd()) return null;
    const tk = peek();
    if (!tk) return null;
    if (tk.type === 'identifier' && tk.value === 'media') return parseMediaRule();
    if (tk.type === 'punctuation' && tk.value === ':') return parsePseudoRule();
    return parseDeclaration();
  }
  function parseDeclaration() {
    const prop = consumeIdentifier('Esperado nome de propriedade (identifier).');
    consumeExactPunctuation(':', `Esperado ':' após '${prop.value}'.`);
    const valueTokens = [];
    while (!isAtEnd()) {
      const next = peek();
      if (!next) break;
      if (next.type === 'punctuation') {
        if (next.value === ';' || next.value === '}' || next.value === '{') break;
      }
      const tk = consume();
      if (tk) valueTokens.push(tk);
    }
    const maybeEnd = peek();
    if (maybeEnd && maybeEnd.type === 'punctuation' && maybeEnd.value === ';') consume();
    const valueStr = stringifyValue(valueTokens);
    return {
      type: 'Declaration',
      property: prop.value,
      value: valueStr
    };
  }
  function parseMediaRule() {
    consume(); // media
    consumeExactPunctuation('(', "Esperado '(' após 'media'.");
    const condition = parseMediaCondition();
    consumeExactPunctuation(')', "Esperado ')' após condição de mídia.");
    consumeExactPunctuation('{', "Esperado '{' após media(...).");
    const declarations = [];
    while (!isAtEnd()) {
      if (matchPunctuation('}')) {
        consume();
        break;
      }
      skipPunctuation(';');
      declarations.push(parseDeclaration());
    }
    skipPunctuation(';');
    return {
      type: 'MediaRule',
      condition,
      declarations
    };
  }
  function parseMediaCondition() {
    const prop = consumeIdentifier('Esperado identificador (ex.: max-width) na condição de mídia.');
    consumeExactPunctuation(':', "Esperado ':' na condição de mídia.");
    const numberToken = consumeNumber('Esperado número em media condition, ex.: 768px.');
    return {
      type: 'MediaCondition',
      property: prop.value,
      value: {
        type: 'Number',
        value: parseFloat(numberToken.value),
        unit: numberToken.unit
      }
    };
  }
  function parsePseudoRule() {
    consume(); // ':'
    const pseudo = consumeIdentifier("Esperado identificador após ':' (ex.: 'hover').");
    consumeExactPunctuation('{', `Esperado '{' após :${pseudo.value}.`);
    const declarations = [];
    while (!isAtEnd()) {
      if (matchPunctuation('}')) {
        consume();
        break;
      }
      skipPunctuation(';');
      declarations.push(parseDeclaration());
    }
    skipPunctuation(';');
    return {
      type: 'PseudoRule',
      pseudo: pseudo.value,
      declarations
    };
  }

  /*************************************************
   * Helpers
   *************************************************/
  function skipPunctuation(symbol) {
    while (!isAtEnd()) {
      const tk = peek();
      if (!tk) break;
      if (tk.type === 'punctuation' && tk.value === symbol) consume();else break;
    }
  }
  function matchPunctuation(symbol) {
    if (isAtEnd()) return false;
    const tk = peek();
    return !!tk && tk.type === 'punctuation' && tk.value === symbol;
  }
  function consumeExactPunctuation(symbol, errMsg) {
    const tk = consume();
    if (!tk || tk.type !== 'punctuation' || tk.value !== symbol) throw new Error(`${errMsg} Encontrado: ${JSON.stringify(tk)}`);
  }
  function consumeIdentifier(errMsg) {
    const tk = consume();
    if (!tk || tk.type !== 'identifier') throw new Error(`${errMsg} Encontrado: ${JSON.stringify(tk)}`);
    return tk;
  }
  function consumeNumber(errMsg) {
    const tk = consume();
    if (!tk || tk.type !== 'number') throw new Error(`${errMsg} Encontrado: ${JSON.stringify(tk)}`);
    return tk;
  }
  function tokenToString(tk) {
    return tk.type === 'number' ? tk.value + (tk.unit || '') : tk.value;
  }
  function stringifyValue(tokens) {
    let result = '';
    for (let i = 0; i < tokens.length; i++) {
      const tk = tokens[i];
      if (!tk) break;
      const part = tokenToString(tk);
      if (tk.type === 'punctuation') {
        if (part === '(' || part === ')' || part === ',') {
          result += part;
        } else {
          result += part;
        }
      } else {
        result += part;
      }
      const next = tokens[i + 1];
      if (part === ',') {
        result += ' ';
      } else if ((tk.type === 'identifier' || tk.type === 'number' || part === ')') && next) {
        if (next.type === 'identifier' || next.type === 'number') {
          result += ' ';
        }
      }
    }
    return result;
  }
}
//# sourceMappingURL=parseAST.js.map