"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.tokenizeCSS = tokenizeCSS;
function tokenizeCSS(cssString) {
  const tokens = [];
  const len = cssString.length;
  let current = 0;

  // Tabelas de lookup para verificações rápidas
  const punctuationMap = {
    '{': 1,
    '}': 1,
    ':': 1,
    ';': 1,
    ',': 1,
    '(': 1,
    ')': 1
  };
  const whitespaceMap = {
    ' ': 1,
    '\n': 1,
    '\t': 1,
    '\r': 1,
    '\f': 1
  };
  const digitMap = {
    '0': 1,
    '1': 1,
    '2': 1,
    '3': 1,
    '4': 1,
    '5': 1,
    '6': 1,
    '7': 1,
    '8': 1,
    '9': 1
  };

  // Buffer para construção de strings (mais rápido que concatenação)
  const stringBuffer = new Array(128);
  let bufferIndex = 0;
  function clearBuffer() {
    bufferIndex = 0;
  }
  function bufferToString() {
    return stringBuffer.slice(0, bufferIndex).join('');
  }
  while (current < len) {
    const char = cssString[current];

    // Whitespace - verificação direta sem regex
    if (whitespaceMap[char]) {
      current++;
      continue;
    }

    // Comentários de bloco /* */
    if (char === '/' && cssString[current + 1] === '*') {
      current += 2;
      while (current < len) {
        if (cssString[current] === '*' && cssString[current + 1] === '/') {
          current += 2;
          break;
        }
        current++;
      }
      continue;
    }

    // Comentários de linha //
    if (char === '/' && cssString[current + 1] === '/') {
      current += 2;
      while (current < len && cssString[current] !== '\n') {
        current++;
      }
      continue;
    }

    // Tokens de pontuação - verificação direta por tabela
    if (punctuationMap[char]) {
      tokens.push({
        type: 'punctuation',
        value: char
      });
      current++;
      continue;
    }

    // Strings com aspas
    if (char === '"' || char === "'") {
      const quote = char;
      stringBuffer[bufferIndex++] = char;
      current++;
      while (current < len) {
        const c = cssString[current++];
        stringBuffer[bufferIndex++] = c;
        if (c === '\\' && current < len) {
          // Escape sequence
          stringBuffer[bufferIndex++] = cssString[current++];
        } else if (c === quote) {
          break;
        }
      }
      tokens.push({
        type: 'string',
        value: bufferToString()
      });
      clearBuffer();
      continue;
    }

    // Números (incluindo decimais e unidades)
    if (digitMap[char] || char === '.' && digitMap[cssString[current + 1]]) {
      // Parte numérica
      while (current < len && (digitMap[cssString[current]] || cssString[current] === '.')) {
        stringBuffer[bufferIndex++] = cssString[current++];
      }
      const value = bufferToString();
      clearBuffer();

      // Unidade (px, em, %, etc)
      const startUnit = current;
      while (current < len) {
        const c = cssString[current];
        // Fast check for letter characters without regex
        const code = c.charCodeAt(0);
        if (code >= 97 && code <= 122 ||
        // a-z
        code >= 65 && code <= 90 ||
        // A-Z
        c === '-' || c === '_' || c === '%') {
          current++;
        } else {
          break;
        }
      }
      const unit = current > startUnit ? cssString.slice(startUnit, current) : '';
      tokens.push({
        type: 'number',
        value,
        ...(unit ? {
          unit
        } : {})
      });
      continue;
    }

    // Identificadores (seletores, propriedades, valores)
    if (char >= 'a' && char <= 'z' || char >= 'A' && char <= 'Z' || char === '_' || char === '-' || char === '#' || char === '.') {
      const start = current;
      while (current < len) {
        const c = cssString[current];
        if (whitespaceMap[c] || punctuationMap[c]) {
          break;
        }
        current++;
      }
      const value = cssString.slice(start, current);

      // Determinar tipo eficientemente
      let type = 'identifier';
      if (value.charCodeAt(0) === 35) {
        // '#'
        type = 'color';
      } else if (value.charCodeAt(0) === 46 ||
      // '.'
      value.charCodeAt(0) === 35 ||
      // '#'
      value === '*') {
        type = 'selector';
      }
      tokens.push({
        type,
        value
      });
      continue;
    }

    // Avançar por caracteres não reconhecidos
    current++;
  }
  return tokens;
}

// const styled = (templateString: TemplateStringsArray) => {
//   const cssString = templateString.join('').trim();

//   const tokens = tokenizeCSS(cssString);

//   //   console.log(tokens);

//   const ast = parseTokensToAST(tokens);

//   const unistyles = astToUnistyles(ast);
// };

// styled`
//   background-color: red;
//   color: blue;
//   font-size: 16px;
//   font-weight: bold;
//   text-align: center;
//   padding: 10px;
//   border-radius: 5px;
//   border: 1px solid black;

//   @media (max-width: 768px) {
//     background-color: green;
//   };

//   &:hover {
//     background-color: yellow;
//   };

//   &:active {
//     background-color: red;
//   };
// `;
//# sourceMappingURL=tokenizer.js.map