"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.camelCase = camelCase;
exports.default = plugin;
var _traverse = _interopRequireDefault(require("@babel/traverse"));
var _types = require("@babel/types");
var _functional = require("./helpers/functional.js");
var _parseAST = require("./helpers/parseAST.js");
var _astToUnistyles = require("./helpers/ast-to-unistyles.js");
var _tokenizer = require("./helpers/tokenizer.js");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
// ---------------------------------------------------------
// Logger helper
// ---------------------------------------------------------
const debug = (...args) => {
  console.log(new Date().toISOString(), '[unistyles-styled-opt]', ...args);
};

/* --------------------------------------------------------- */
/* utilidades                                                */
/* --------------------------------------------------------- */

function hasUseStyled(path) {
  debug('hasUseStyled: checking directives');
  (0, _functional.Functional)(path.node.directives).some('use styled').then(() => {
    path.state = {
      useStyled: true
    };
    debug('hasUseStyled: found directive, state.useStyled=true');
  });
}
function createStaticComponent(baseElement, componentName, propsExpr) {
  debug('createStaticComponent:', componentName);
  const args = [baseElement, (0, _types.memberExpression)((0, _types.identifier)('styles'), (0, _types.identifier)(componentName.toLowerCase()))];
  if (propsExpr) {
    args.push(propsExpr);
    debug('createStaticComponent: propsExpr provided');
  }
  const node = (0, _types.callExpression)((0, _types.memberExpression)((0, _types.identifier)('styled'), (0, _types.identifier)('createComponent')), args);
  debug('createStaticComponent: created node', node.type);
  return node;
}
function isCssIdentifier(node) {
  const result = (0, _types.isIdentifier)(node, {
    name: 'css'
  }) || (0, _types.isMemberExpression)(node) && (0, _types.isIdentifier)(node.property, {
    name: 'css'
  });
  debug('isCssIdentifier:', result);
  return result;
}
function convertCssTaggedTemplate(tt) {
  debug('convertCssTaggedTemplate: static css``');
  const strings = tt.quasi.quasis.map(q => q.value.raw);
  const obj = parseTemplateArrayString(strings);
  debug('convertCssTaggedTemplate: parsed to object', obj);
  return (0, _types.valueToNode)(obj);
}
function convertCssCall(call) {
  debug('convertCssCall: entering');
  if (call.arguments.length === 1 && (0, _types.isTemplateLiteral)(call.arguments[0])) {
    const tl = call.arguments[0];
    if (tl.expressions.length === 0) {
      debug('convertCssCall: static template literal');
      const strings = tl.quasis.map(q => q.value.raw);
      const obj = parseTemplateArrayString(strings);
      debug('convertCssCall: parsed to object', obj);
      return (0, _types.valueToNode)(obj);
    }
  }
  debug('convertCssCall: no conversion applied');
  return null;
}
function camelCase(input) {
  debug('camelCase: input', input);
  const words = input.match(/[A-Za-z0-9]+/g) ?? [];
  const result = words.map((w, i) => i === 0 ? w.toLowerCase() : w.charAt(0).toUpperCase() + w.slice(1).toLowerCase()).join('');
  debug('camelCase: output', result);
  return result;
}
function convertCssTaggedTemplateDynamic(tt) {
  debug('convertCssTaggedTemplateDynamic: entering');
  const {
    quasis,
    expressions
  } = tt.quasi;
  if (!expressions.length) {
    debug('convertCssTaggedTemplateDynamic: no expressions');
    return null;
  }
  const props = [];
  quasis.forEach((q, i) => {
    const before = q.value.raw;
    const expr = expressions[i];
    if (!expr || !(0, _types.isMemberExpression)(expr)) return;
    let base = expr;
    while ((0, _types.isMemberExpression)(base.object)) base = base.object;
    if (!(0, _types.isIdentifier)(base.object, {
      name: 'theme'
    })) return;
    const m = /([\w-]+)\s*:\s*$/.exec(before);
    if (!m) return;
    const cssProp = m[1];
    const jsProp = camelCase(cssProp);
    props.push((0, _types.objectProperty)((0, _types.identifier)(jsProp), expr));
    debug('convertCssTaggedTemplateDynamic: added dynamic prop', jsProp);
  });
  const result = props.length ? (0, _types.objectExpression)(props) : null;
  debug('convertCssTaggedTemplateDynamic: result', result);
  return result;
}
function parseTemplateArrayString(templateArrayString) {
  debug('parseTemplateArrayString: strings', templateArrayString);
  const cssString = templateArrayString.join('').trim();
  if (!cssString) {
    debug('parseTemplateArrayString: empty CSS');
    return {};
  }
  const tokens = (0, _tokenizer.tokenizeCSS)(cssString);
  const ast = (0, _parseAST.parseTokensToAST)(tokens);
  const result = (0, _astToUnistyles.astToUnistyles)(ast);
  debug('parseTemplateArrayString: AST to unistyles', result);
  return JSON.parse(JSON.stringify(result));
}
function ensureStyleSheetImport(path) {
  debug('ensureStyleSheetImport: checking import');
  const program = path.findParent(p => p.isProgram());
  if (program && !program.scope.hasBinding('StyleSheet')) {
    debug('ensureStyleSheetImport: adding import');
    const imp = (0, _types.importDeclaration)([(0, _types.importSpecifier)((0, _types.identifier)('StyleSheet'), (0, _types.identifier)('StyleSheet'))], (0, _types.stringLiteral)('react-native-unistyles'));
    program.unshiftContainer('body', imp);
  }
}
function spreadFromExpr(expr, usesTheme) {
  debug('spreadFromExpr: usesTheme', usesTheme);
  const arg = usesTheme ? (0, _types.objectExpression)([(0, _types.spreadElement)((0, _types.identifier)('props')), (0, _types.objectProperty)((0, _types.identifier)('theme'), (0, _types.identifier)('theme'))]) : (0, _types.identifier)('props');
  if ((0, _types.isArrowFunctionExpression)(expr) || (0, _types.isFunctionExpression)(expr)) {
    debug('spreadFromExpr: arrow/function expression');
    return (0, _types.spreadElement)((0, _types.callExpression)(expr, [arg]));
  }
  if ((0, _types.isCallExpression)(expr)) {
    debug('spreadFromExpr: call expression');
    return (0, _types.spreadElement)(expr);
  }
  debug('spreadFromExpr: default');
  return (0, _types.spreadElement)(expr);
}
function buildDynamicFn(staticObj, exprs, usesTheme, linkedProps) {
  debug('buildDynamicFn: staticObj keys', Object.keys(staticObj));
  debug('buildDynamicFn: exprs count', exprs.length, 'usesTheme', usesTheme);
  const propsArray = [(0, _types.spreadElement)((0, _types.valueToNode)(staticObj))];
  exprs.forEach((e, idx) => {
    const link = linkedProps.get(idx);
    if (link) {
      const {
        propName,
        arg
      } = link;
      debug('buildDynamicFn: linked prop', propName);
      const val = (0, _types.isArrowFunctionExpression)(e) || (0, _types.isFunctionExpression)(e) ? (0, _types.callExpression)(e, [arg]) : e;
      propsArray.push((0, _types.objectProperty)((0, _types.identifier)(propName), val));
    } else {
      propsArray.push(spreadFromExpr(e, usesTheme));
    }
  });
  const fn = (0, _types.arrowFunctionExpression)([(0, _types.identifier)('props')], (0, _types.objectExpression)(propsArray));
  debug('buildDynamicFn: created arrow function');
  return fn;
}
function createStyleSheetConstant(stylesObjectAst, usesTheme) {
  debug('createStyleSheetConstant: usesTheme', usesTheme);
  const arg = usesTheme ? (0, _types.arrowFunctionExpression)([(0, _types.identifier)('theme')], stylesObjectAst) : stylesObjectAst;
  const decl = (0, _types.variableDeclaration)('const', [(0, _types.variableDeclarator)((0, _types.identifier)('styles'), (0, _types.callExpression)((0, _types.memberExpression)((0, _types.identifier)('StyleSheet'), (0, _types.identifier)('create')), [arg]))]);
  debug('createStyleSheetConstant: declaration created');
  return decl;
}
function containsThemeIdentifier(node) {
  let found = false;
  (0, _traverse.default)(node, {
    noScope: true,
    Identifier(p) {
      if (p.node.name === 'theme') {
        found = true;
        p.stop();
      }
    }
  });
  debug('containsThemeIdentifier:', found);
  return found;
}

/* --------------------------------------------------------- */
/* plugin principal                                          */
/* --------------------------------------------------------- */

function plugin() {
  return {
    name: 'unistyles-styled-opt',
    visitor: {
      Program(path) {
        debug('visitor.Program: start');
        hasUseStyled(path);
      },
      TaggedTemplateExpression(path) {
        debug('visitor.TaggedTemplateExpression: start');
        // 1. Static css`` conversion
        path.traverse({
          TaggedTemplateExpression(inner) {
            if (!isCssIdentifier(inner.node.tag)) return;
            debug('Nested TaggedTemplateExpression: css identifier');
            if (inner.node.quasi.expressions.length === 0) {
              debug('Static css tagged template detected');
              inner.replaceWith(convertCssTaggedTemplate(inner.node));
              return;
            }
            const dyn = convertCssTaggedTemplateDynamic(inner.node);
            if (dyn) {
              debug('Dynamic css tagged template with theme');
              inner.replaceWith(dyn);
            }
          },
          CallExpression(inner) {
            if (isCssIdentifier(inner.node.callee)) {
              debug('Nested CallExpression: css call');
              const repl = convertCssCall(inner.node);
              if (repl) {
                debug('Replacing css call with object node');
                inner.replaceWith(repl);
              }
            }
          }
        });

        // 2. Identify styled target and attrs
        let propsNode = null;
        let styledTarget = path.node.tag;
        if ((0, _types.isCallExpression)(styledTarget) && (0, _types.isMemberExpression)(styledTarget.callee) && (0, _types.isIdentifier)(styledTarget.callee.property, {
          name: 'attrs'
        })) {
          if (styledTarget.arguments.length) {
            propsNode = (0, _types.cloneNode)(styledTarget.arguments[0], true);
            debug('Found attrs props', propsNode?.type);
          }
          styledTarget = styledTarget.callee.object;
        }

        // 3. Determine baseElement
        let baseElementNode = null;
        if ((0, _types.isMemberExpression)(styledTarget) && (0, _types.isIdentifier)(styledTarget.object, {
          name: 'styled'
        }) && (0, _types.isIdentifier)(styledTarget.property)) {
          baseElementNode = (0, _types.stringLiteral)(styledTarget.property.name);
        } else if ((0, _types.isCallExpression)(styledTarget) && (0, _types.isIdentifier)(styledTarget.callee, {
          name: 'styled'
        }) && styledTarget.arguments.length === 1) {
          baseElementNode = (0, _types.cloneNode)(styledTarget.arguments[0], true);
        }
        if (!baseElementNode) {
          debug('No baseElement detected, skipping');
          return;
        }
        debug('Base element:', baseElementNode.type);

        // 4. Split static / dynamic
        const {
          quasis,
          expressions: rawExprs
        } = path.node.quasi;
        const staticObj = parseTemplateArrayString(quasis.map(q => q.value.raw));
        debug('Parsed staticObj keys', Object.keys(staticObj));
        const exprs = rawExprs.slice();

        // Link props
        const linkedProps = new Map();
        const usesTheme = exprs.some(e => containsThemeIdentifier(e));
        debug('usesTheme in main:', usesTheme);
        quasis.forEach((q, i) => {
          const before = q.value.raw;
          const m = /([\w-]+)\s*:\s*$/.exec(before);
          if (!m) return;
          const jsProp = camelCase(m[1]);
          delete staticObj[jsProp];
          const arg = usesTheme ? (0, _types.objectExpression)([(0, _types.spreadElement)((0, _types.identifier)('props')), (0, _types.objectProperty)((0, _types.identifier)('theme'), (0, _types.identifier)('theme'))]) : (0, _types.identifier)('props');
          linkedProps.set(i, {
            propName: jsProp,
            arg
          });
          debug('Linked prop in main', jsProp);
        });
        const hasDynamic = exprs.length > 0;

        // 5. Ensure styles binding
        const varDecl = path.findParent(p => p.isVariableDeclarator());
        if (!varDecl || !(0, _types.isVariableDeclarator)(varDecl.node) || !(0, _types.isIdentifier)(varDecl.node.id)) {
          debug('No variableDeclarator parent, skipping');
          return;
        }
        const styleKey = varDecl.node.id.name.toLowerCase();
        debug('styleKey:', styleKey);
        const programPath = path.findParent(p => p.isProgram());
        let binding = programPath?.scope.getBinding('styles');
        if (!binding) {
          debug('No styles binding, creating new');
          ensureStyleSheetImport(path);
          const initDecl = createStyleSheetConstant((0, _types.objectExpression)([]), usesTheme);
          varDecl.getStatementParent().insertBefore(initDecl);
          programPath?.scope.crawl();
          binding = programPath?.scope.getBinding('styles');
        } else if (usesTheme) {
          debug('Styles exists but needs theme wrapper');
          const initPath = binding.path.get('init');
          if (initPath.isCallExpression()) {
            const [arg] = initPath.get('arguments');
            if (arg?.isObjectExpression()) {
              const wrapper = (0, _types.arrowFunctionExpression)([(0, _types.identifier)('theme')], arg.node);
              arg.replaceWith(wrapper);
              debug('Wrapped existing styles init with theme');
            }
          }
        }
        if (!binding) return;

        // 6. Locate styles object
        const initPath = binding.path.get('init');
        if (!initPath.isCallExpression()) return;
        const [argPathRaw] = initPath.get('arguments');
        let sheetObjPath = null;
        if (argPathRaw?.isObjectExpression()) {
          sheetObjPath = argPathRaw;
        } else if (argPathRaw?.isArrowFunctionExpression()) {
          const body = argPathRaw.get('body');
          if (body.isObjectExpression()) sheetObjPath = body;
        }
        if (!sheetObjPath) return;

        // 7. Inject styleKey if missing
        const exists = sheetObjPath.node.properties.some(p => (0, _types.isIdentifier)(p.key) && p.key.name === styleKey);
        if (!exists) {
          debug('Injecting new styleKey:', styleKey);
          const valNode = hasDynamic ? buildDynamicFn(staticObj, exprs, usesTheme, linkedProps) : (0, _types.valueToNode)(staticObj);
          sheetObjPath.pushContainer('properties', (0, _types.objectProperty)((0, _types.identifier)(styleKey), valNode));
        }

        // 8. Replace component init
        const varInit = varDecl.get('init');
        let replacement;
        if (hasDynamic) {
          debug('Building dynamic component replacement');
          const dynamicFn = (0, _types.arrowFunctionExpression)([(0, _types.identifier)('props')], (0, _types.callExpression)((0, _types.memberExpression)((0, _types.identifier)('styles'), (0, _types.identifier)(styleKey)), [(0, _types.identifier)('props')]));
          const args = [baseElementNode, dynamicFn];
          if (propsNode) args.push(propsNode);
          replacement = (0, _types.callExpression)((0, _types.memberExpression)((0, _types.identifier)('styled'), (0, _types.identifier)('createComponent')), args);
        } else {
          debug('Building static component replacement');
          replacement = createStaticComponent(baseElementNode, styleKey, propsNode);
        }
        varInit.replaceWith(replacement);
        debug('visitor.TaggedTemplateExpression: end for', styleKey);
      }
    }
  };
}
//# sourceMappingURL=index.js.map