"use strict";

import traverse from '@babel/traverse';
import { arrowFunctionExpression, callExpression, cloneNode, identifier, importDeclaration, importSpecifier, isArrowFunctionExpression, isCallExpression, isFunctionExpression, isIdentifier, isMemberExpression, isTemplateLiteral, isVariableDeclarator, memberExpression, objectExpression, objectProperty, spreadElement, stringLiteral, valueToNode, variableDeclaration, variableDeclarator as declarator } from '@babel/types';
import { Functional } from "./helpers/functional.js";
import { parseTokensToAST } from "./helpers/parseAST.js";
import { astToUnistyles } from "./helpers/ast-to-unistyles.js";
import { tokenizeCSS } from "./helpers/tokenizer.js";

// ---------------------------------------------------------
// Logger helper
// ---------------------------------------------------------
const debug = (...args) => {
  console.log(new Date().toISOString(), '[unistyles-styled-opt]', ...args);
};

/* --------------------------------------------------------- */
/* utilidades                                                */
/* --------------------------------------------------------- */

function hasUseStyled(path) {
  debug('hasUseStyled: checking directives');
  Functional(path.node.directives).some('use styled').then(() => {
    path.state = {
      useStyled: true
    };
    debug('hasUseStyled: found directive, state.useStyled=true');
  });
}
function createStaticComponent(baseElement, componentName, propsExpr) {
  debug('createStaticComponent:', componentName);
  const args = [baseElement, memberExpression(identifier('styles'), identifier(componentName.toLowerCase()))];
  if (propsExpr) {
    args.push(propsExpr);
    debug('createStaticComponent: propsExpr provided');
  }
  const node = callExpression(memberExpression(identifier('styled'), identifier('createComponent')), args);
  debug('createStaticComponent: created node', node.type);
  return node;
}
function isCssIdentifier(node) {
  const result = isIdentifier(node, {
    name: 'css'
  }) || isMemberExpression(node) && isIdentifier(node.property, {
    name: 'css'
  });
  debug('isCssIdentifier:', result);
  return result;
}
function convertCssTaggedTemplate(tt) {
  debug('convertCssTaggedTemplate: static css``');
  const strings = tt.quasi.quasis.map(q => q.value.raw);
  const obj = parseTemplateArrayString(strings);
  debug('convertCssTaggedTemplate: parsed to object', obj);
  return valueToNode(obj);
}
function convertCssCall(call) {
  debug('convertCssCall: entering');
  if (call.arguments.length === 1 && isTemplateLiteral(call.arguments[0])) {
    const tl = call.arguments[0];
    if (tl.expressions.length === 0) {
      debug('convertCssCall: static template literal');
      const strings = tl.quasis.map(q => q.value.raw);
      const obj = parseTemplateArrayString(strings);
      debug('convertCssCall: parsed to object', obj);
      return valueToNode(obj);
    }
  }
  debug('convertCssCall: no conversion applied');
  return null;
}
export function camelCase(input) {
  debug('camelCase: input', input);
  const words = input.match(/[A-Za-z0-9]+/g) ?? [];
  const result = words.map((w, i) => i === 0 ? w.toLowerCase() : w.charAt(0).toUpperCase() + w.slice(1).toLowerCase()).join('');
  debug('camelCase: output', result);
  return result;
}
function convertCssTaggedTemplateDynamic(tt) {
  debug('convertCssTaggedTemplateDynamic: entering');
  const {
    quasis,
    expressions
  } = tt.quasi;
  if (!expressions.length) {
    debug('convertCssTaggedTemplateDynamic: no expressions');
    return null;
  }
  const props = [];
  quasis.forEach((q, i) => {
    const before = q.value.raw;
    const expr = expressions[i];
    if (!expr || !isMemberExpression(expr)) return;
    let base = expr;
    while (isMemberExpression(base.object)) base = base.object;
    if (!isIdentifier(base.object, {
      name: 'theme'
    })) return;
    const m = /([\w-]+)\s*:\s*$/.exec(before);
    if (!m) return;
    const cssProp = m[1];
    const jsProp = camelCase(cssProp);
    props.push(objectProperty(identifier(jsProp), expr));
    debug('convertCssTaggedTemplateDynamic: added dynamic prop', jsProp);
  });
  const result = props.length ? objectExpression(props) : null;
  debug('convertCssTaggedTemplateDynamic: result', result);
  return result;
}
function parseTemplateArrayString(templateArrayString) {
  debug('parseTemplateArrayString: strings', templateArrayString);
  const cssString = templateArrayString.join('').trim();
  if (!cssString) {
    debug('parseTemplateArrayString: empty CSS');
    return {};
  }
  const tokens = tokenizeCSS(cssString);
  const ast = parseTokensToAST(tokens);
  const result = astToUnistyles(ast);
  debug('parseTemplateArrayString: AST to unistyles', result);
  return JSON.parse(JSON.stringify(result));
}
function ensureStyleSheetImport(path) {
  debug('ensureStyleSheetImport: checking import');
  const program = path.findParent(p => p.isProgram());
  if (program && !program.scope.hasBinding('StyleSheet')) {
    debug('ensureStyleSheetImport: adding import');
    const imp = importDeclaration([importSpecifier(identifier('StyleSheet'), identifier('StyleSheet'))], stringLiteral('react-native-unistyles'));
    program.unshiftContainer('body', imp);
  }
}
function spreadFromExpr(expr, usesTheme) {
  debug('spreadFromExpr: usesTheme', usesTheme);
  const arg = usesTheme ? objectExpression([spreadElement(identifier('props')), objectProperty(identifier('theme'), identifier('theme'))]) : identifier('props');
  if (isArrowFunctionExpression(expr) || isFunctionExpression(expr)) {
    debug('spreadFromExpr: arrow/function expression');
    return spreadElement(callExpression(expr, [arg]));
  }
  if (isCallExpression(expr)) {
    debug('spreadFromExpr: call expression');
    return spreadElement(expr);
  }
  debug('spreadFromExpr: default');
  return spreadElement(expr);
}
function buildDynamicFn(staticObj, exprs, usesTheme, linkedProps) {
  debug('buildDynamicFn: staticObj keys', Object.keys(staticObj));
  debug('buildDynamicFn: exprs count', exprs.length, 'usesTheme', usesTheme);
  const propsArray = [spreadElement(valueToNode(staticObj))];
  exprs.forEach((e, idx) => {
    const link = linkedProps.get(idx);
    if (link) {
      const {
        propName,
        arg
      } = link;
      debug('buildDynamicFn: linked prop', propName);
      const val = isArrowFunctionExpression(e) || isFunctionExpression(e) ? callExpression(e, [arg]) : e;
      propsArray.push(objectProperty(identifier(propName), val));
    } else {
      propsArray.push(spreadFromExpr(e, usesTheme));
    }
  });
  const fn = arrowFunctionExpression([identifier('props')], objectExpression(propsArray));
  debug('buildDynamicFn: created arrow function');
  return fn;
}
function createStyleSheetConstant(stylesObjectAst, usesTheme) {
  debug('createStyleSheetConstant: usesTheme', usesTheme);
  const arg = usesTheme ? arrowFunctionExpression([identifier('theme')], stylesObjectAst) : stylesObjectAst;
  const decl = variableDeclaration('const', [declarator(identifier('styles'), callExpression(memberExpression(identifier('StyleSheet'), identifier('create')), [arg]))]);
  debug('createStyleSheetConstant: declaration created');
  return decl;
}
function containsThemeIdentifier(node) {
  let found = false;
  traverse(node, {
    noScope: true,
    Identifier(p) {
      if (p.node.name === 'theme') {
        found = true;
        p.stop();
      }
    }
  });
  debug('containsThemeIdentifier:', found);
  return found;
}

/* --------------------------------------------------------- */
/* plugin principal                                          */
/* --------------------------------------------------------- */

export default function plugin() {
  return {
    name: 'unistyles-styled-opt',
    visitor: {
      Program(path) {
        debug('visitor.Program: start');
        hasUseStyled(path);
      },
      TaggedTemplateExpression(path) {
        debug('visitor.TaggedTemplateExpression: start');
        // 1. Static css`` conversion
        path.traverse({
          TaggedTemplateExpression(inner) {
            if (!isCssIdentifier(inner.node.tag)) return;
            debug('Nested TaggedTemplateExpression: css identifier');
            if (inner.node.quasi.expressions.length === 0) {
              debug('Static css tagged template detected');
              inner.replaceWith(convertCssTaggedTemplate(inner.node));
              return;
            }
            const dyn = convertCssTaggedTemplateDynamic(inner.node);
            if (dyn) {
              debug('Dynamic css tagged template with theme');
              inner.replaceWith(dyn);
            }
          },
          CallExpression(inner) {
            if (isCssIdentifier(inner.node.callee)) {
              debug('Nested CallExpression: css call');
              const repl = convertCssCall(inner.node);
              if (repl) {
                debug('Replacing css call with object node');
                inner.replaceWith(repl);
              }
            }
          }
        });

        // 2. Identify styled target and attrs
        let propsNode = null;
        let styledTarget = path.node.tag;
        if (isCallExpression(styledTarget) && isMemberExpression(styledTarget.callee) && isIdentifier(styledTarget.callee.property, {
          name: 'attrs'
        })) {
          if (styledTarget.arguments.length) {
            propsNode = cloneNode(styledTarget.arguments[0], true);
            debug('Found attrs props', propsNode?.type);
          }
          styledTarget = styledTarget.callee.object;
        }

        // 3. Determine baseElement
        let baseElementNode = null;
        if (isMemberExpression(styledTarget) && isIdentifier(styledTarget.object, {
          name: 'styled'
        }) && isIdentifier(styledTarget.property)) {
          baseElementNode = stringLiteral(styledTarget.property.name);
        } else if (isCallExpression(styledTarget) && isIdentifier(styledTarget.callee, {
          name: 'styled'
        }) && styledTarget.arguments.length === 1) {
          baseElementNode = cloneNode(styledTarget.arguments[0], true);
        }
        if (!baseElementNode) {
          debug('No baseElement detected, skipping');
          return;
        }
        debug('Base element:', baseElementNode.type);

        // 4. Split static / dynamic
        const {
          quasis,
          expressions: rawExprs
        } = path.node.quasi;
        const staticObj = parseTemplateArrayString(quasis.map(q => q.value.raw));
        debug('Parsed staticObj keys', Object.keys(staticObj));
        const exprs = rawExprs.slice();

        // Link props
        const linkedProps = new Map();
        const usesTheme = exprs.some(e => containsThemeIdentifier(e));
        debug('usesTheme in main:', usesTheme);
        quasis.forEach((q, i) => {
          const before = q.value.raw;
          const m = /([\w-]+)\s*:\s*$/.exec(before);
          if (!m) return;
          const jsProp = camelCase(m[1]);
          delete staticObj[jsProp];
          const arg = usesTheme ? objectExpression([spreadElement(identifier('props')), objectProperty(identifier('theme'), identifier('theme'))]) : identifier('props');
          linkedProps.set(i, {
            propName: jsProp,
            arg
          });
          debug('Linked prop in main', jsProp);
        });
        const hasDynamic = exprs.length > 0;

        // 5. Ensure styles binding
        const varDecl = path.findParent(p => p.isVariableDeclarator());
        if (!varDecl || !isVariableDeclarator(varDecl.node) || !isIdentifier(varDecl.node.id)) {
          debug('No variableDeclarator parent, skipping');
          return;
        }
        const styleKey = varDecl.node.id.name.toLowerCase();
        debug('styleKey:', styleKey);
        const programPath = path.findParent(p => p.isProgram());
        let binding = programPath?.scope.getBinding('styles');
        if (!binding) {
          debug('No styles binding, creating new');
          ensureStyleSheetImport(path);
          const initDecl = createStyleSheetConstant(objectExpression([]), usesTheme);
          varDecl.getStatementParent().insertBefore(initDecl);
          programPath?.scope.crawl();
          binding = programPath?.scope.getBinding('styles');
        } else if (usesTheme) {
          debug('Styles exists but needs theme wrapper');
          const initPath = binding.path.get('init');
          if (initPath.isCallExpression()) {
            const [arg] = initPath.get('arguments');
            if (arg?.isObjectExpression()) {
              const wrapper = arrowFunctionExpression([identifier('theme')], arg.node);
              arg.replaceWith(wrapper);
              debug('Wrapped existing styles init with theme');
            }
          }
        }
        if (!binding) return;

        // 6. Locate styles object
        const initPath = binding.path.get('init');
        if (!initPath.isCallExpression()) return;
        const [argPathRaw] = initPath.get('arguments');
        let sheetObjPath = null;
        if (argPathRaw?.isObjectExpression()) {
          sheetObjPath = argPathRaw;
        } else if (argPathRaw?.isArrowFunctionExpression()) {
          const body = argPathRaw.get('body');
          if (body.isObjectExpression()) sheetObjPath = body;
        }
        if (!sheetObjPath) return;

        // 7. Inject styleKey if missing
        const exists = sheetObjPath.node.properties.some(p => isIdentifier(p.key) && p.key.name === styleKey);
        if (!exists) {
          debug('Injecting new styleKey:', styleKey);
          const valNode = hasDynamic ? buildDynamicFn(staticObj, exprs, usesTheme, linkedProps) : valueToNode(staticObj);
          sheetObjPath.pushContainer('properties', objectProperty(identifier(styleKey), valNode));
        }

        // 8. Replace component init
        const varInit = varDecl.get('init');
        let replacement;
        if (hasDynamic) {
          debug('Building dynamic component replacement');
          const dynamicFn = arrowFunctionExpression([identifier('props')], callExpression(memberExpression(identifier('styles'), identifier(styleKey)), [identifier('props')]));
          const args = [baseElementNode, dynamicFn];
          if (propsNode) args.push(propsNode);
          replacement = callExpression(memberExpression(identifier('styled'), identifier('createComponent')), args);
        } else {
          debug('Building static component replacement');
          replacement = createStaticComponent(baseElementNode, styleKey, propsNode);
        }
        varInit.replaceWith(replacement);
        debug('visitor.TaggedTemplateExpression: end for', styleKey);
      }
    }
  };
}
//# sourceMappingURL=index.js.map