import { createElement, type ComponentProps, type ComponentType } from 'react';
import {
  ActivityIndicator,
  Button,
  FlatList,
  Image,
  ImageBackground,
  KeyboardAvoidingView,
  Modal,
  Pressable,
  RefreshControl,
  SafeAreaView,
  ScrollView,
  SectionList,
  StatusBar,
  Switch,
  Text,
  TextInput,
  TouchableHighlight,
  TouchableOpacity,
  TouchableWithoutFeedback,
  View,
  VirtualizedList,
} from 'react-native';
import { StyleSheet } from 'react-native-unistyles';

/* ------------------------------------------------------------------ */
/* tipagens utilitárias                                               */
/* ------------------------------------------------------------------ */

interface CssStyle extends TemplateStringsArray {}

/** Pedaços da tag-template: primeiro array de strings + …expressions */
type ArgsComponentStyle<T extends unknown[]> = [CssStyle, ...T];

type Primitive = string | number | object | null | undefined | false;
/** Função de interpolação que enxerga `theme` (e eventuais props) */
type CssInterpolation<P> =
  | Primitive
  | ((props: P & { theme: any }) => Primitive);
/* ------------------------------------------------------------------ */
/* styled                                                             */
/* ------------------------------------------------------------------ */

function WithStyled<C extends ComponentType<any>>(component: C) {
  const base = <K,>(
    ..._: ArgsComponentStyle<CssInterpolation<ComponentProps<C> & K>[]>
  ) => component as ComponentType<ComponentProps<C> & K>;

  base.attrs = function (attrs: ComponentProps<C>) {
    return (_: CssStyle) => createElement(component, attrs) as unknown as C;
  };

  return base;
}

export function styled<T extends ComponentType<any>>(component?: T) {
  return WithStyled<T>(component as T);
}

/* ------------------------------------------------------------------ */
/* css – agora tipado e levando theme                                 */
/* ------------------------------------------------------------------ */

export function css<P = {}>(
  ..._: ArgsComponentStyle<CssInterpolation<P>[]>
): CssInterpolation<P> {
  // marcador para o plugin Babel; substituído em build-time
  return null as unknown as CssInterpolation<P>;
}

/* ------------------------------------------------------------------ */
/* extras                                                             */
/* ------------------------------------------------------------------ */

styled.attrs = function (attrs: any) {
  return (component: any) => createElement(component, attrs);
};

styled.View = WithStyled(View);
styled.Text = WithStyled(Text);
styled.FlatList = WithStyled(FlatList);
styled.SectionList = WithStyled(SectionList);
styled.VirtualizedList = WithStyled(VirtualizedList);
styled.Pressable = WithStyled(Pressable);
styled.ScrollView = WithStyled(ScrollView);
styled.KeyboardAvoidingView = WithStyled(KeyboardAvoidingView);
styled.SafeAreaView = WithStyled(SafeAreaView);
styled.Image = WithStyled(Image);
styled.ImageBackground = WithStyled(ImageBackground);
styled.TextInput = WithStyled(TextInput);
styled.ActivityIndicator = WithStyled(ActivityIndicator);
styled.Button = WithStyled(Button);
styled.Modal = WithStyled(Modal);
styled.StatusBar = WithStyled(StatusBar);
styled.Switch = WithStyled(Switch);
styled.TouchableHighlight = WithStyled(TouchableHighlight);
styled.TouchableOpacity = WithStyled(TouchableOpacity);
styled.TouchableWithoutFeedback = WithStyled(TouchableWithoutFeedback);
styled.RefreshControl = WithStyled(RefreshControl);

type ComponentKeys =
  | 'View'
  | 'Text'
  | 'FlatList'
  | 'SectionList'
  | 'VirtualizedList'
  | 'Pressable'
  | 'ScrollView'
  | 'KeyboardAvoidingView'
  | 'SafeAreaView'
  | 'Image'
  | 'ImageBackground'
  | 'TextInput'
  | 'ActivityIndicator'
  | 'Button'
  | 'Modal'
  | 'StatusBar'
  | 'Switch'
  | 'TouchableHighlight'
  | 'TouchableOpacity'
  | 'TouchableWithoutFeedback'
  | 'RefreshControl';

styled.createComponent = function (component: string, styles: any) {
  return ({ children, style: componentStyle, ...rest }: any) => {
    const dynamicStyle =
      typeof styles === 'function' ? styles(rest) : styles || {};
    const style = StyleSheet.flatten([componentStyle, dynamicStyle]);

    const styledComponent = styled[component as ComponentKeys];

    return createElement(
      // @ts-ignore – resolvido em runtime
      styledComponent ? styledComponent`` : component,
      { ...rest, style },
      children
    );
  };
};

export { default as plugin } from './plugin';
