import { type PluginObj, type PluginPass, type NodePath } from '@babel/core';
import traverse from '@babel/traverse';
import {
  arrowFunctionExpression,
  callExpression,
  cloneNode,
  identifier,
  importDeclaration,
  importSpecifier,
  isArrowFunctionExpression,
  isCallExpression,
  isFunctionExpression,
  isIdentifier,
  isMemberExpression,
  isTemplateLiteral,
  isVariableDeclarator,
  memberExpression,
  objectExpression,
  objectProperty,
  spreadElement,
  stringLiteral,
  type Expression,
  type Program,
  valueToNode,
  variableDeclaration,
  variableDeclarator as declarator,
  type TaggedTemplateExpression,
  type ObjectExpression,
  type ObjectProperty,
  type SpreadElement,
} from '@babel/types';
import { Functional } from './helpers/functional';
import { parseTokensToAST } from './helpers/parseAST';
import { astToUnistyles } from './helpers/ast-to-unistyles';
import { tokenizeCSS } from './helpers/tokenizer';

// ---------------------------------------------------------
// Logger helper
// ---------------------------------------------------------
const debug = (...args: any[]) => {
  console.log(new Date().toISOString(), '[unistyles-styled-opt]', ...args);
};

/* --------------------------------------------------------- */
/* utilidades                                                */
/* --------------------------------------------------------- */

function hasUseStyled(path: NodePath<Program>) {
  debug('hasUseStyled: checking directives');
  Functional(path.node.directives)
    .some('use styled')
    .then(() => {
      path.state = { useStyled: true };
      debug('hasUseStyled: found directive, state.useStyled=true');
    });
}

function createStaticComponent(
  baseElement: Expression,
  componentName: string,
  propsExpr: Expression | null
) {
  debug('createStaticComponent:', componentName);
  const args: Expression[] = [
    baseElement,
    memberExpression(
      identifier('styles'),
      identifier(componentName.toLowerCase())
    ),
  ];
  if (propsExpr) {
    args.push(propsExpr);
    debug('createStaticComponent: propsExpr provided');
  }
  const node = callExpression(
    memberExpression(identifier('styled'), identifier('createComponent')),
    args
  );
  debug('createStaticComponent: created node', node.type);
  return node;
}

function isCssIdentifier(node: Expression): boolean {
  const result =
    isIdentifier(node, { name: 'css' }) ||
    (isMemberExpression(node) && isIdentifier(node.property, { name: 'css' }));
  debug('isCssIdentifier:', result);
  return result;
}

function convertCssTaggedTemplate(tt: TaggedTemplateExpression) {
  debug('convertCssTaggedTemplate: static css``');
  const strings = tt.quasi.quasis.map((q) => q.value.raw);
  const obj = parseTemplateArrayString(strings);
  debug('convertCssTaggedTemplate: parsed to object', obj);
  return valueToNode(obj);
}

function convertCssCall(call: any) {
  debug('convertCssCall: entering');
  if (call.arguments.length === 1 && isTemplateLiteral(call.arguments[0])) {
    const tl = call.arguments[0];
    if (tl.expressions.length === 0) {
      debug('convertCssCall: static template literal');
      const strings = tl.quasis.map((q: any) => q.value.raw);
      const obj = parseTemplateArrayString(strings);
      debug('convertCssCall: parsed to object', obj);
      return valueToNode(obj);
    }
  }
  debug('convertCssCall: no conversion applied');
  return null;
}

export function camelCase(input: string): string {
  debug('camelCase: input', input);
  const words = input.match(/[A-Za-z0-9]+/g) ?? [];
  const result = words
    .map((w, i) =>
      i === 0
        ? w.toLowerCase()
        : w.charAt(0).toUpperCase() + w.slice(1).toLowerCase()
    )
    .join('');
  debug('camelCase: output', result);
  return result;
}

function convertCssTaggedTemplateDynamic(tt: TaggedTemplateExpression) {
  debug('convertCssTaggedTemplateDynamic: entering');
  const { quasis, expressions } = tt.quasi;
  if (!expressions.length) {
    debug('convertCssTaggedTemplateDynamic: no expressions');
    return null;
  }

  const props: ObjectProperty[] = [];
  quasis.forEach((q, i) => {
    const before = q.value.raw;
    const expr = expressions[i];
    if (!expr || !isMemberExpression(expr)) return;

    let base: any = expr;
    while (isMemberExpression(base.object)) base = base.object;
    if (!isIdentifier(base.object, { name: 'theme' })) return;

    const m = /([\w-]+)\s*:\s*$/.exec(before);
    if (!m) return;
    const cssProp = m[1];
    const jsProp = camelCase(cssProp as any);
    props.push(objectProperty(identifier(jsProp), expr));
    debug('convertCssTaggedTemplateDynamic: added dynamic prop', jsProp);
  });

  const result = props.length ? objectExpression(props) : null;
  debug('convertCssTaggedTemplateDynamic: result', result);
  return result;
}

function parseTemplateArrayString(templateArrayString: string[]) {
  debug('parseTemplateArrayString: strings', templateArrayString);
  const cssString = templateArrayString.join('').trim();
  if (!cssString) {
    debug('parseTemplateArrayString: empty CSS');
    return {};
  }
  const tokens = tokenizeCSS(cssString);
  const ast = parseTokensToAST(tokens as any);
  const result = astToUnistyles(ast);
  debug('parseTemplateArrayString: AST to unistyles', result);
  return JSON.parse(JSON.stringify(result));
}

function ensureStyleSheetImport(path: NodePath<Program>) {
  debug('ensureStyleSheetImport: checking import');
  const program = path.findParent((p) => p.isProgram());
  if (program && !program.scope.hasBinding('StyleSheet')) {
    debug('ensureStyleSheetImport: adding import');
    const imp = importDeclaration(
      [importSpecifier(identifier('StyleSheet'), identifier('StyleSheet'))],
      stringLiteral('react-native-unistyles')
    );
    program.unshiftContainer('body' as never, imp as any);
  }
}

function spreadFromExpr(expr: Expression, usesTheme: boolean) {
  debug('spreadFromExpr: usesTheme', usesTheme);
  const arg = usesTheme
    ? objectExpression([
        spreadElement(identifier('props') as any),
        objectProperty(identifier('theme'), identifier('theme')),
      ])
    : identifier('props');

  if (isArrowFunctionExpression(expr) || isFunctionExpression(expr)) {
    debug('spreadFromExpr: arrow/function expression');
    return spreadElement(callExpression(expr as any, [arg]));
  }
  if (isCallExpression(expr)) {
    debug('spreadFromExpr: call expression');
    return spreadElement(expr);
  }
  debug('spreadFromExpr: default');
  return spreadElement(expr);
}

function buildDynamicFn(
  staticObj: any,
  exprs: Expression[],
  usesTheme: boolean,
  linkedProps: any
) {
  debug('buildDynamicFn: staticObj keys', Object.keys(staticObj));
  debug('buildDynamicFn: exprs count', exprs.length, 'usesTheme', usesTheme);
  const propsArray: (ObjectProperty | SpreadElement)[] = [
    spreadElement(valueToNode(staticObj) as Expression),
  ];

  exprs.forEach((e, idx) => {
    const link = linkedProps.get(idx);
    if (link) {
      const { propName, arg } = link;
      debug('buildDynamicFn: linked prop', propName);
      const val =
        isArrowFunctionExpression(e) || isFunctionExpression(e)
          ? callExpression(e as any, [arg])
          : e;
      propsArray.push(objectProperty(identifier(propName), val) as any);
    } else {
      propsArray.push(spreadFromExpr(e, usesTheme));
    }
  });

  const fn = arrowFunctionExpression(
    [identifier('props')],
    objectExpression(propsArray) as any
  );
  debug('buildDynamicFn: created arrow function');
  return fn;
}

function createStyleSheetConstant(
  stylesObjectAst: ObjectExpression,
  usesTheme: boolean
) {
  debug('createStyleSheetConstant: usesTheme', usesTheme);
  const arg = usesTheme
    ? arrowFunctionExpression([identifier('theme')], stylesObjectAst)
    : stylesObjectAst;

  const decl = variableDeclaration('const', [
    declarator(
      identifier('styles'),
      callExpression(
        memberExpression(identifier('StyleSheet'), identifier('create')),
        [arg]
      )
    ),
  ]);
  debug('createStyleSheetConstant: declaration created');
  return decl;
}

function containsThemeIdentifier(node: Expression): boolean {
  let found = false;
  traverse(node, {
    noScope: true,
    Identifier(p) {
      if (p.node.name === 'theme') {
        found = true;
        p.stop();
      }
    },
  });
  debug('containsThemeIdentifier:', found);
  return found;
}

/* --------------------------------------------------------- */
/* plugin principal                                          */
/* --------------------------------------------------------- */

export default function plugin(): PluginObj<PluginPass> {
  return {
    name: 'unistyles-styled-opt',
    visitor: {
      Program(path) {
        debug('visitor.Program: start');
        hasUseStyled(path as any);
      },

      TaggedTemplateExpression(path) {
        debug('visitor.TaggedTemplateExpression: start');
        // 1. Static css`` conversion
        path.traverse({
          TaggedTemplateExpression(inner) {
            if (!isCssIdentifier(inner.node.tag as Expression)) return;
            debug('Nested TaggedTemplateExpression: css identifier');
            if (inner.node.quasi.expressions.length === 0) {
              debug('Static css tagged template detected');
              inner.replaceWith(convertCssTaggedTemplate(inner.node));
              return;
            }
            const dyn = convertCssTaggedTemplateDynamic(inner.node as any);
            if (dyn) {
              debug('Dynamic css tagged template with theme');
              inner.replaceWith(dyn);
            }
          },
          CallExpression(inner) {
            if (isCssIdentifier(inner.node.callee as Expression)) {
              debug('Nested CallExpression: css call');
              const repl = convertCssCall(inner.node);
              if (repl) {
                debug('Replacing css call with object node');
                inner.replaceWith(repl);
              }
            }
          },
        });

        // 2. Identify styled target and attrs
        let propsNode: Expression | null = null;
        let styledTarget = path.node.tag;
        if (
          isCallExpression(styledTarget) &&
          isMemberExpression(styledTarget.callee) &&
          isIdentifier(styledTarget.callee.property, { name: 'attrs' })
        ) {
          if (styledTarget.arguments.length) {
            propsNode = cloneNode(
              styledTarget.arguments[0] as Expression,
              true
            );
            debug('Found attrs props', propsNode?.type);
          }
          styledTarget = styledTarget.callee.object;
        }

        // 3. Determine baseElement
        let baseElementNode: Expression | null = null;
        if (
          isMemberExpression(styledTarget) &&
          isIdentifier(styledTarget.object, { name: 'styled' }) &&
          isIdentifier(styledTarget.property)
        ) {
          baseElementNode = stringLiteral(styledTarget.property.name);
        } else if (
          isCallExpression(styledTarget) &&
          isIdentifier(styledTarget.callee, { name: 'styled' }) &&
          styledTarget.arguments.length === 1
        ) {
          baseElementNode = cloneNode(
            styledTarget.arguments[0] as Expression,
            true
          );
        }
        if (!baseElementNode) {
          debug('No baseElement detected, skipping');
          return;
        }
        debug('Base element:', baseElementNode.type);

        // 4. Split static / dynamic
        const { quasis, expressions: rawExprs } = path.node.quasi;
        const staticObj = parseTemplateArrayString(
          quasis.map((q) => q.value.raw)
        );
        debug('Parsed staticObj keys', Object.keys(staticObj));
        const exprs = rawExprs.slice();

        // Link props
        const linkedProps = new Map<
          number,
          { propName: string; arg: Expression }
        >();
        const usesTheme = exprs.some((e) => containsThemeIdentifier(e as any));
        debug('usesTheme in main:', usesTheme);
        quasis.forEach((q, i) => {
          const before = q.value.raw;
          const m = /([\w-]+)\s*:\s*$/.exec(before);
          if (!m) return;
          const jsProp = camelCase(m[1] as any);
          delete staticObj[jsProp];
          const arg = usesTheme
            ? objectExpression([
                spreadElement(identifier('props') as any),
                objectProperty(identifier('theme'), identifier('theme')),
              ])
            : identifier('props');
          linkedProps.set(i, { propName: jsProp, arg });
          debug('Linked prop in main', jsProp);
        });
        const hasDynamic = exprs.length > 0;

        // 5. Ensure styles binding
        const varDecl = path.findParent((p) => p.isVariableDeclarator());
        if (
          !varDecl ||
          !isVariableDeclarator(varDecl.node) ||
          !isIdentifier(varDecl.node.id)
        ) {
          debug('No variableDeclarator parent, skipping');
          return;
        }
        const styleKey = varDecl.node.id.name.toLowerCase();
        debug('styleKey:', styleKey);
        const programPath = path.findParent((p) => p.isProgram());
        let binding = programPath?.scope.getBinding('styles');

        if (!binding) {
          debug('No styles binding, creating new');
          ensureStyleSheetImport(path as any);
          const initDecl = createStyleSheetConstant(
            objectExpression([]),
            usesTheme
          );
          (varDecl.getStatementParent() as any).insertBefore(initDecl);
          programPath?.scope.crawl();
          binding = programPath?.scope.getBinding('styles');
        } else if (usesTheme) {
          debug('Styles exists but needs theme wrapper');
          const initPath = binding.path.get('init');
          if (initPath.isCallExpression()) {
            const [arg] = initPath.get<'arguments'>('arguments');
            if (arg?.isObjectExpression()) {
              const wrapper = arrowFunctionExpression(
                [identifier('theme')],
                arg.node
              );
              arg.replaceWith(wrapper);
              debug('Wrapped existing styles init with theme');
            }
          }
        }
        if (!binding) return;

        // 6. Locate styles object
        const initPath = binding.path.get('init');
        if (!initPath.isCallExpression()) return;
        const [argPathRaw] = initPath.get<'arguments'>('arguments');
        let sheetObjPath: NodePath<ObjectExpression> | null = null;
        if (argPathRaw?.isObjectExpression()) {
          sheetObjPath = argPathRaw;
        } else if (argPathRaw?.isArrowFunctionExpression()) {
          const body = argPathRaw.get('body');
          if (body.isObjectExpression()) sheetObjPath = body;
        }
        if (!sheetObjPath) return;

        // 7. Inject styleKey if missing
        const exists = sheetObjPath.node.properties.some(
          (p: any) => isIdentifier(p.key) && p.key.name === styleKey
        );
        if (!exists) {
          debug('Injecting new styleKey:', styleKey);
          const valNode = hasDynamic
            ? buildDynamicFn(
                staticObj,
                exprs as Expression[],
                usesTheme,
                linkedProps
              )
            : valueToNode(staticObj);
          sheetObjPath.pushContainer(
            'properties',
            objectProperty(identifier(styleKey), valNode)
          );
        }

        // 8. Replace component init
        const varInit = (varDecl as any).get('init');
        let replacement: Expression;
        if (hasDynamic) {
          debug('Building dynamic component replacement');
          const dynamicFn = arrowFunctionExpression(
            [identifier('props')],
            callExpression(
              memberExpression(identifier('styles'), identifier(styleKey)),
              [identifier('props')]
            )
          );
          const args: Expression[] = [baseElementNode, dynamicFn];
          if (propsNode) args.push(propsNode);
          replacement = callExpression(
            memberExpression(
              identifier('styled'),
              identifier('createComponent')
            ),
            args
          );
        } else {
          debug('Building static component replacement');
          replacement = createStaticComponent(
            baseElementNode,
            styleKey,
            propsNode
          );
        }
        varInit.replaceWith(replacement);
        debug('visitor.TaggedTemplateExpression: end for', styleKey);
      },
    },
  };
}
